@@ -1,3 +1,4 @@
+#!/usr/bin/python
 # -*- coding: utf-8 -*-
 ## Copyright (C) 2001-2008 Alberto Milone <albertomilone@alice.it>
 
@@ -18,10 +19,8 @@
 
 import gtk, gobject, sys, dbus, logging
 import ScreenResolution
-from ScreenResolution.policykit import PolicyKitAuthentication
 from ScreenResolution import ui
 
-POLICY_KIT_ACTION = 'com.ubuntu.screenresolution.mechanism.configure'
 SERVICE_NAME   = 'com.ubuntu.ScreenResolution.Mechanism'
 OBJECT_PATH    = '/'
 INTERFACE_NAME = 'com.ubuntu.ScreenResolution.Mechanism'
@@ -37,6 +36,8 @@ from XKit import xutils, xorgparser
 clean = False
 
 def checkVirtual(virtres):
+    '''See if it's necessary to set the virtual resolution'''
+
     source = '/etc/X11/xorg.conf'
     
     try:
@@ -45,9 +46,7 @@ def checkVirtual(virtres):
         return False
     
     if len(a.globaldict['Screen']) > 0:
-        '''
-        See if the virtual resolution is already there and if it's big enough
-        '''
+        # See if the virtual resolution is already there and if it's big enough
         res = None
         for screen in a.globaldict['Screen']:
             try:
@@ -59,67 +58,17 @@ def checkVirtual(virtres):
                         res = res.lower().split(' ')
                         res = filter(lambda x: x != '', res)
                         if len(res) == 2 and int(virtres[0]) <= int(res[0]) and int(virtres[1]) <= int(res[1]):
-                            '''
-                            Nothing to do, the virtual resolution is already there
-                            '''
+                            # Nothing to do, the virtual resolution is already there
                             return True
             except (XKit.xorgparser.SectionException, XKit.xorgparser.OptionException, AttributeError):
                 pass
     
     return False
 
-def computeVirtual(*params):
-    '''
-    Compute the virtual resolution on the basis of the
-    position and resolution of the screens.
-    '''
-    xResolutions = []
-    yResolutions = []
-    for param in params[0]:
-        position = param[: str(param).find(':')].split(',')
-        relativeResolution = param[param.find(':') +1:].split('x')
-        xres = int(position[0]) + int(relativeResolution[0])
-        yres = int(position[1]) + int(relativeResolution[1])
-        xResolutions.append(xres)
-        yResolutions.append(yres)
-    
-    return (max(xResolutions), max(yResolutions))
-
-def compareFrameBuffer(virtual):
-    x = virtual[0]
-    y = virtual[1]
-    p1 = Popen(['xrandr', '-q'], stdout=PIPE)
-    p = p1.communicate()[0]
-    a = p.split('\n')
-    b = a[0].split(',')
-    c = b[2].strip().split(' ')
-    c.remove('maximum')
-    c.remove('x')
-    xfb = int(c[0])
-    yfb = int(c[1])
-    if x > xfb or y > yfb:
-        '''
-        Call the GUI with policykit
-        '''
-        #Set the Virtual Resolution to str(x), str(y)
-        return [True, (str(x), str(y))]
-    else:
-        #Virtual Resolution is not required
-        return [False]
-
-
 def get_xkit_service(widget=None):
     '''
     returns a dbus interface to the screenresolution mechanism
     '''
-    policy_auth = PolicyKitAuthentication()    
-
-    granted = policy_auth.obtain_authorization(POLICY_KIT_ACTION, widget)
-    logging.debug("granted = %s" % granted)
-
-    if not granted:
-        return None
-    
     service_object = dbus.SystemBus().get_object(SERVICE_NAME, OBJECT_PATH)
     service = dbus.Interface(service_object, INTERFACE_NAME)
 
@@ -175,6 +124,8 @@ def gui_dialog(message, parent_dialog,
 
 
 class BootWindow:
+    optimal_virtual_resolution = ['2048', '2048']
+    
     def __init__(self, resolution):
         
         translation = ui.AbstractUI()
@@ -226,6 +177,14 @@ class BootWindow:
 #            gui_dialog(self.dbus_cant_connect,
 #                              self.window, message_type='error')
             sys.exit(1)
+        
+        # If the required resolution is lower than the optimal virtual
+        # resolution (usually the highest resolution which doesn't break
+        # direct rendering) then use the latter instead.
+        if int(self.resolution[0]) < int(self.optimal_virtual_resolution[0]) and \
+           int(self.resolution[1]) < int(self.optimal_virtual_resolution[1]):
+            self.resolution = self.optimal_virtual_resolution
+
         status = self.conf.setVirtual(self.resolution)
         #print 'Status', status
         if status == True:
@@ -277,4 +236,3 @@ if __name__ == "__main__":
     gtk.main()
     if not clean:
         sys.exit(1)
-    #call policy_kit
