@@ -25,11 +25,6 @@
 #
 # modified by Alberto Milone (tseliot) <albertomilone@alice.it>
 
-'''
-Processes, which want to access this service have to acquire first
-the action id "com.ubuntu.xkit.mechanism.configure" from PolicyKit
-'''
-
 import dbus, dbus.service, gobject, logging
 import os, os.path
 import sys
@@ -38,71 +33,6 @@ import time
 import shutil
 import subprocess
 
-POLICY_KIT_ACTION     = 'com.ubuntu.screenresolution.mechanism.configure'
-class PolicyKitAuthentication(object):
-    '''
-    Helper class to do authentication via PolicyKit
-    '''
-    def __init__(self):
-        super(PolicyKitAuthentication, self).__init__()
-        self.__pk = None
-        self.__pa = None
-
-    def is_authorized(self, action_id, pid=None):
-        '''
-        Ask PolicyKit whether we are already authorized.
-        '''
-
-        # Check whether the process is authorized:
-        if pid == None:
-            pid = os.getpid()
-            
-        pid = dbus.UInt32(pid)
-        authorized = self.policy_kit.IsProcessAuthorized(action_id, pid, False)
-        logging.debug('%s: authorized=%r', action_id, authorized)
-
-        return ('yes' == authorized)
-
-    def obtain_authorization(self, action_id, widget=None):
-        '''
-        Try to obtain authoriztation for the specified action.
-        '''
-        if self.is_authorized(action_id):
-            return True
-
-        xid = (widget and widget.get_toplevel().window.xid or 0)
-        xid, pid = dbus.UInt32(xid), dbus.UInt32(os.getpid())
-
-        granted = self.auth_agent.ObtainAuthorization(action_id, xid, pid)
-        logging.debug('%s: granted=%r', action_id, granted)
-
-        return bool(granted)
-
-    def __get_policy_kit(self):
-        '''Retreive the D-Bus interface of PolicyKit.'''
-
-        if self.__pk:
-            return self.__pk
-
-        # retreiving the interface raises DBusException on error:
-        service = dbus.SystemBus().get_object('org.freedesktop.PolicyKit', '/')
-        self.__pk = dbus.Interface(service, 'org.freedesktop.PolicyKit')
-        return self.__pk        
-
-    def __get_auth_agent(self):
-        "Retreive the D-Bus interface of the PolicyKit authentication agent."
-
-        if self.__pa:
-            return self.__pa
-        # retreiving the interface raises DBusException on error:
-        self.__pa = dbus.SessionBus().get_object(
-            'org.freedesktop.PolicyKit.AuthenticationAgent', '/')
-        return self.__pa
-
-    auth_agent = property(__get_auth_agent)
-    policy_kit = property(__get_policy_kit)
-
-
 # Modern flavors of dbus bindings have that symbol in dbus.lowlevel,
 # for old flavours the internal _dbus_bindings module must be used.
 
@@ -131,54 +61,42 @@ class UsageError(dbus.DBusException):
 class PolicyKitService(dbus.service.Object):
     '''A D-BUS service that uses PolicyKit for authorization.'''
 
-    def _check_permission(self, sender, action=POLICY_KIT_ACTION):
+    def _check_permission(self, sender, conn, action, reconnect=True):
         '''
         Verifies if the specified action is permitted, and raises
         an AccessDeniedException if not.
-
-        The caller should use ObtainAuthorization() to get permission.
-
-        Needs:
-        /usr/bin/polkit-auth --user root --grant org.freedesktop.policykit.read
-
         '''
-        if not sender:
-            raise AccessDeniedException('Session not authorized by PolicyKit')
+        if sender is None and conn is None:
+            # called locally, not through D-BUS
+            return
 
+        # get peer PID
+        dbus_info = dbus.Interface(conn.get_object('org.freedesktop.DBus',
+              '/org/freedesktop/DBus/Bus', False), 'org.freedesktop.DBus')
+        pid = dbus_info.GetConnectionUnixProcessID(sender)
+
+        # query PolicyKit
+        polkit = dbus.Interface(get_service_bus().get_object(
+                'org.freedesktop.PolicyKit1',
+                '/org/freedesktop/PolicyKit1/Authority', False),
+                'org.freedesktop.PolicyKit1.Authority')
         try:
-            policy_auth = PolicyKitAuthentication()
-            bus = dbus.SystemBus()
-            dbus_object = bus.get_object('org.freedesktop.DBus', '/')
-            dbus_object = dbus.Interface(dbus_object, 'org.freedesktop.DBus')
+            # we don't need is_challenge return here, since we call with AllowUserInteraction
+            (is_auth, _, details) = polkit.CheckAuthorization(
+                    ('unix-process', {'pid': dbus.UInt32(pid, variant_level=1)}),
+                    action, {'': ''}, dbus.UInt32(1), '', timeout=600)
+        except dbus.DBusException, e:
+            if reconnect and e._dbus_error_name == 'org.freedesktop.DBus.Error.ServiceUnknown':
+                # polkitd timed out, connect again
+                return self._check_polkit_privilege(sender, conn, action, reconnect=False)
+            else:
+                raise
 
-            kit = bus.get_object('org.freedesktop.PolicyKit', '/')
-            kit = dbus.Interface(kit, 'org.freedesktop.PolicyKit')
-            pid = dbus.UInt32(dbus_object.GetConnectionUnixProcessID(sender))
+        if not is_auth:
+            logging.debug('_check_permission: sender %s on connection %s pid %i is not authorized for %s: %s' %
+                    (sender, conn, pid, action, str(details)))
+            raise AccessDeniedException(action)
 
-            granted = policy_auth.is_authorized(action, pid)
-            logging.info('process authorization: %r', granted)
-            print >> sys.stderr,  'process authorization: %r' % granted
-
-            if not granted:
-                raise AccessDeniedException('Process not authorized'
-                                            ' by PolicyKit')
-
-            granted = policy_auth.policy_kit.IsSystemBusNameAuthorized(action,
-                                                                       sender,
-                                                                       False)
-            logging.info('authorization of system bus name: %r', granted)
-
-            if 'yes' != granted:
-                raise AccessDeniedException('Session not '
-                                            'authorized by PolicyKit')
-
-        except AccessDeniedException:
-            logging.info("AccessDeniedException")
-            raise
-
-        except dbus.DBusException, ex:
-            logging.info("AccessDeniedException %r",  ex)
-            raise AccessDeniedException(ex.message)
 
 class BackendService(PolicyKitService):
     '''A D-Bus service that PolicyKit controls access to.'''
@@ -246,11 +164,14 @@ class BackendService(PolicyKitService):
     # pylint: disable-msg=R0913
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='as', out_signature='b',
-                         sender_keyword='sender')   
-    def setVirtual(self, virtres, sender=None):
+                         sender_keyword='sender', connection_keyword='conn')   
+    def setVirtual(self, virtres, sender=None, conn=None):
         '''
         Replace the first line of this example with a source and a destination file
         '''
+        self._check_permission(sender, conn,
+            'com.ubuntu.screenresolution.mechanism.configure')
+
         virtual = ' '.join(virtres)
         
         source = '/etc/X11/xorg.conf'
@@ -310,10 +231,14 @@ class BackendService(PolicyKitService):
     
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='s', out_signature='i',
-                         sender_keyword='sender')   
-    def setDontZap(self, enable, sender=None):
+                         sender_keyword='sender', connection_keyword='conn')
+    def setDontZap(self, enable, sender=None, conn=None):
         '''Try to set the DontZap option in the xorg.conf
            and return the exit code of dontzap'''
+
+        self._check_permission(sender, conn,
+            'com.ubuntu.screenresolution.mechanism.dontzap')
+
         dontzap_file = '/usr/bin/dontzap'
         if not os.path.isfile(dontzap_file):
             logging.error('/usr/bin/dontzap does not exist')
